/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name : battery.c
* Version : 1.0
* Device(s) : R7F0C001
* H/W Platform : YCB15R7F0C001001B
* Description : This file implements device driver for battery.
******************************************************************************/
/******************************************************************************
* History : DD.MM.YYYY Version Description
* : 20.09.2014 1.00 First Release
******************************************************************************/

/******************************************************************************
Includes <System Includes> , <Project Includes>
******************************************************************************/
#include "userdefine.h"
#include "battery.h"
#include "adc.h"
#include "lcd.h"
#include "timer.h"
#include "monitor.h"
#include "beep.h"

/******************************************************************************
Exported global variables and functions(can be accessed by other files)
******************************************************************************/
uint8_t g_fChargeConnected;		/* Charge line connected flag */
uint8_t g_ChargeStatus;			/* Charge status */
uint16_t g_fBatteryFlashCounter;	/* Control the flash time of battery flag */
uint16_t g_aSupplyVoltageBuf[10];	/* Save the voltage of VDD from ADC */
uint8_t g_SupplyVoltageSampleCounter;	/* Simple counter of VDD */
uint16_t g_BatterVoltage;		/* Save the voltage of battery */
uint8_t g_BatteryCapacityIndicator;	/* Battery capacity indicator */
uint8_t g_BatteryFlagStep;		/* Control the display step when battery is charging */

extern uint8_t g_MainMode;
extern uint8_t g_fSleepModeOn;
extern uint8_t g_fLowPowerSleepOn;
extern uint8_t g_fNewDataDispRq;
extern uint8_t g_fSensorStopReq;
extern uint8_t g_fAlarmOnFlashOn;
extern uint8_t g_fSensorSimpling;
extern uint8_t g_fBatteryFlagDispRq;
extern uint16_t g_ModeTimeOutCounter;
extern uint8_t g_OutLowPowerModeCounter;

static void Get_Supply_Voltage(void);
/***********************************************************************************************************************
* Function Name: Get_Supply_Voltage
* Description  : This function gets the voltage value of battery
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void Get_Supply_Voltage(void)
{
	
	/* Get the voltage of supply power */
	if(0U == g_fSensorSimpling)
	{
		g_aSupplyVoltageBuf[g_SupplyVoltageSampleCounter++] = (ADC_Sample(BATTERY_ANI_CH,5)*2);	/*select ANI0 as input and sample time is 5 times*/
		if(g_SupplyVoltageSampleCounter >= 10U)
		{
			g_SupplyVoltageSampleCounter = 0;
		}
	}
}
/***********************************************************************************************************************
End of function Get_Supply_Voltage
***********************************************************************************************************************/
/******************************************************************************
* Function Name : Check_Charge_Status
* Description : This function gets battery charge status.
* Arguments : none
* Return Value : none
******************************************************************************/	
void Check_Charge_Status(void)
{
	/* Check the status of charging */
	if(0U == g_fSensorSimpling)
	{
		/* get the voltage of charging input */
		/* Select ANI20 as input and sample time is 5 times */
		if(ADC_Sample(CHARGE_LINE_ANI_CH,5) > AD_CHARGE_ON_VOLTAGE_VALUE)
		{
			if(0U == g_fChargeConnected)
			{
				g_fChargeConnected = 1;
			}
			if(((ADC_Sample(BATTERY_ANI_CH,3)*2) >= (AD_BAT_MAX_VALUE-AD_BAT_CONPENSATION_VALUE))&&(0U == P1.3))
			{
				g_ChargeStatus = STANDBY;
			}
			else
			{
				g_ChargeStatus = CHARGING;
			}	
		}
		else
		{
			if(1U == g_fChargeConnected)
			{
				g_fChargeConnected = 0;
				g_ChargeStatus = DISCONNECED;
			}
		}
	}
}
/******************************************************************************
End of function Check_Charge_Status 
******************************************************************************/


/******************************************************************************
* Function Name : Battery_Process
* Description : This function implements the battery process.
* Arguments : none
* Return Value : none
******************************************************************************/	
void Battery_Process(void)
{
	/* Battery process */
	Get_Supply_Voltage();
	if(9U == g_SupplyVoltageSampleCounter)
	{
		g_BatterVoltage = g_aSupplyVoltageBuf[9];
		if(g_MainMode == BATTERY_MODE)
		{
			g_fNewDataDispRq = ON;
		}
	}
	if(g_fBatteryFlashCounter > 0)
	{
		g_fBatteryFlashCounter--;
		
	}
	else
	{
		/* The battery indicators flash from left to right and back again when charging */
		if(g_ChargeStatus == CHARGING)
		{
			g_BatteryFlagStep++;
			if(g_BatteryFlagStep>4)
			{
				g_BatteryFlagStep = 0;
			}
			g_BatteryCapacityIndicator = g_BatteryFlagStep;
		}
		else
		{
			g_BatteryFlagStep = 0;
			g_BatteryCapacityIndicator = (uint8_t)(((g_BatterVoltage - AD_BAT_MIN_VALUE)/AD_BAT_STEP_VALUE)+1);
		}
		g_fBatteryFlashCounter = 500/MAIN_LOOP_TIME; /* 500ms */
		g_fBatteryFlagDispRq = ON;
	}
	if((g_BatterVoltage < LOW_POWER_VOLTAGE_VALUE)&&(g_MainMode != INITIAL_MODE)&&(g_MainMode != LOW_POWER_MODE))
	{
		/* Meet the condition to enter the low power mode */
		GoTo_LowPowerMode();
	}
}
/******************************************************************************
End of function Battery_Process
******************************************************************************/

/***********************************************************************************************************************
* Function Name: GoTo_BatteryMode
* Description  : This function does some preparative works before goto the battery mode
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void GoTo_BatteryMode(void)
{
	if(g_MainMode == ALARM_ON_MODE)
	{
		Alarm_Off();
	}
	g_ModeTimeOutCounter = BATTERY_DISP_TIMEOUT_TIME;
	g_fSensorStopReq = ON;
	BATTERY_EN = ON;
	BATTEEY_LED = ON;
	g_fAlarmOnFlashOn = OFF;
	g_MainMode = BATTERY_MODE;
	g_fNewDataDispRq = ON;
	RTC_Start();
}
/***********************************************************************************************************************
End of function GoTo_BatteryMode
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: GoTo_LowPowerMode
* Description  : This function does some preparative works before goto low power mode
* Arguments    : None 
* Return Value : None
***********************************************************************************************************************/
void GoTo_LowPowerMode(void)
{
	Sensor_Stop();
	g_fLowPowerSleepOn = 1U;
	LCD_Stop();
	BATTERY_EN = OFF;
	BATTEEY_LED = OFF;
	g_MainMode = LOW_POWER_MODE;
	g_OutLowPowerModeCounter = 0;
	RTC_Start();
}
/***********************************************************************************************************************
End of function GoTo_LowPowerMode
***********************************************************************************************************************/