/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name : e2prom.c
* Version : 1.0
* Device(s) : R7F0C001
* H/W Platform : YCB15R7F0C001001B
* Description : This file implements EEPROM access function.
******************************************************************************/
/******************************************************************************
* History : DD.MM.YYYY Version Description
* : 20.09.2015 1.00 First Release
******************************************************************************/


/******************************************************************************
Includes <System Includes> , <Project Includes>
******************************************************************************/
#include "userdefine.h"
#include "e2prom.h"

/******************************************************************************
Macro definitions
******************************************************************************/

#define SCL_INPUT (PM14.4 = 1)       /* Set "scl" as input port */
#define SCL_OUTPUT (PM14.4 = 0)      /* Set "scl" as output port */
#define SCL_LOW (P14.4 = 0)          /* Set "scl" output "0" */
#define SCL_HIGH (P14.4 = 1)         /* Set "scl" output "1" */
#define SCL_INDATA (P14.4)           /* Read "scl" */

#define SDA_INPUT (PM14.3 = 1)       /* Set "sda" as input port */
#define SDA_OUTPUT (PM14.3 = 0)      /* Set "sda" as output port */
#define SDA_LOW (P14.3 = 0)          /* Set "sda" output "0" */
#define SDA_HIGH (P14.3 = 1)         /* Set "sda" output "1" */
#define SDA_INDATA (P14.3)           /* Read "sda" */

/*AT24C64a, 64K SERIAL EEPROM: Internally organized with 256 pages of 32 bytes each, the
64K bit requires an 13-bit data word address for random word addressing.*/

#define EEPROM_BUS_ADDRESS 0xa0U    /* Device address */
#define EEPROM_ADDRESS (0x00U)      /* address = 0, located in first page on EEPROM */
#define EEPROM_PAGE_SIZE 0x20U      /* Each page has 32 bytes data */

#define ACK 0U
#define NACK 1U
/******************************************************************************
Exported global variables and functions(can be accessed by other files)
******************************************************************************/
uint8_t g_aDataBuffer[EEPROM_PAGE_SIZE];    /* EEPROM data buffer */
uint8_t g_fEepromError = FALSE;             /* EEPROM access error flag */

static void Delayus(uint8_t time);
static void I2C_Start(void);
static void I2C_Stop(void);
static uint8_t I2C_WriteByte(uint8_t dat);
static uint8_t I2C_ReadByte(uint8_t ack);
/******************************************************************************
* Function Name:Delayus
* Description : This function implements Delayus function.
* Arguments : uint8_t
* Return Value : none
******************************************************************************/
static void Delayus(uint8_t time)
{   /*CPU clock is 24MHz,wait us*/
    uint8_t i;
    for(i=0; i<time; i++);
    {
	    NOP();
	    NOP();
    }
}
/******************************************************************************
End of Delayus function
******************************************************************************/

/******************************************************************************
* Function Name:Delayms
* Description : This function implements Delayms function.
* Arguments : uint8_t
* Return Value : none
******************************************************************************/
void Delayms(uint8_t time)
{   /*CPU clock is 20MHz, wait ms*/
    uint8_t i;
    for(i=0; i<time; i++)
    {
        Delayus(200);
        Delayus(200);
        Delayus(200);
        Delayus(200);
        Delayus(200);
    }
}
/******************************************************************************
End of Delayms function
******************************************************************************/

/******************************************************************************
* Function Name:I2C_Init
* Description : This function implements I2C_Init function.
* Arguments : none
* Return Value : none
******************************************************************************/
void I2C_Init(void)
{
    /* Initialize SCL and SDA ports */
    POM1 = 0;
    
    SCL_LOW;    
    SCL_OUTPUT;   
    
    SDA_LOW;   
    SDA_OUTPUT;    
         
    Delayus(5); /* Wait 2us */
}
/******************************************************************************
End of I2C_Init function
******************************************************************************/

/******************************************************************************
* Function Name:I2C_Start
* Description : This function implements I2C_Start function.
* Arguments : none
* Return Value : none
******************************************************************************/
static void I2C_Start(void)
{   /* I2C start condition*/
    SDA_OUTPUT;
    SCL_LOW;
    Delayus(2);
    SDA_HIGH;
    Delayus(2);
    SCL_HIGH;
    Delayus(2); 
    SDA_LOW;      
    Delayus(2); 
    SCL_LOW;      
    Delayus(2); 
}
/******************************************************************************
End of I2C_Start function
******************************************************************************/

/******************************************************************************
* Function Name:I2C_Stop
* Description : This function implements I2C_Stop function.
* Arguments : none
* Return Value : none
******************************************************************************/
static void I2C_Stop(void)
{   /* I2C stop condition*/
    SDA_LOW;
    SDA_OUTPUT; 
    Delayus(1);
    SDA_LOW;
    Delayus(2);
    SCL_HIGH;
    Delayus(2); 
    SDA_HIGH;  
    Delayus(2);
    SCL_LOW;
    Delayus(2);
}
/******************************************************************************
End of I2C_Stop function
******************************************************************************/

/******************************************************************************
* Function Name:I2C_WriteByte
* Description : This function implements I2C_WriteByte function.
* Arguments : uint8_t
* Return Value : uint8_t
******************************************************************************/
static uint8_t I2C_WriteByte(uint8_t dat)
{
    /*Write 1byte data to eeprom */
    uint8_t i,result; 
    SDA_OUTPUT;
    for(i = 0;i < 8; i++) 
    {
        SCL_LOW;
        if(dat & 0x80)        
        {
            SDA_HIGH;       
        }
        else
        {
            SDA_LOW;        
        }
                
        Delayus(1);
        SCL_HIGH;        
        dat <<= 1;       
        Delayus(2);  
        SCL_LOW;
        Delayus(1);
    }
    SDA_INPUT;
    Delayus(1);
    SCL_HIGH;       
    if(SDA_INDATA)
    {
        result = 0;     /* Write opration failed flag */    
    }
    else
    {
        result = 1;     /* Write opration success flag */   
    }           
    Delayus(2);
    SCL_LOW;
    return result;      
}

/******************************************************************************
End of I2C_WriteByte function
******************************************************************************/

/******************************************************************************
* Function Name:I2C_ReadByte
* Description : This function implements I2C_ReadByte function.
* Arguments : uint8_t
* Return Value : uint8_t
******************************************************************************/
static uint8_t I2C_ReadByte(uint8_t ack)
{   
    /* Read 1byte data from eeprom */
    uint8_t i,dat = 0;     
    SDA_INPUT; 
    for(i = 0;i < 8;i++)      
    {
        SCL_LOW;          
        Delayus(1);
        SCL_HIGH;
        dat <<= 1;       
        if(SDA_INDATA)
        {
            dat++;  
        }
        Delayus(2);
        SCL_LOW;
        Delayus(1);
    }
    SDA_OUTPUT;
    SCL_LOW;
    Delayus(1);
    if(ack == NACK)     
    {
        SDA_HIGH;   
    }
    else
    {
        SDA_LOW;        
    }
    SCL_HIGH;
    Delayus(2);
    SCL_LOW;
    Delayus(1);
    SDA_HIGH;
    return (dat);     
}
/******************************************************************************
End of I2C_ReadByte function
******************************************************************************/

/******************************************************************************
* Function Name:EEPROM_WritePage
* Description : This function implements EEPROM_WritePage function.
* Arguments : uint16_t
* Return Value : none
******************************************************************************/
void EEPROM_WritePage(uint16_t Address)
{
    /* Write 1 page data to eeprom */
    uint8_t i,flag;
    uint8_t Addr_L,Addr_H;
    Addr_L = (uint8_t)Address;
    Addr_H = (uint8_t)(Address>>8);
    g_fEepromError = 0;
    I2C_Start(); 
    flag = I2C_WriteByte(EEPROM_BUS_ADDRESS);     
    flag = I2C_WriteByte(Addr_H); 
    flag = I2C_WriteByte(Addr_L);
    if (0 == flag)
    {
        g_fEepromError = TRUE;
        I2C_Stop(); 
        return;
            
    }
    for(i = 0; i < EEPROM_PAGE_SIZE; i++)
    {
        flag = I2C_WriteByte(g_aDataBuffer[i]); 
        if (0 == flag)
        {
            g_fEepromError = TRUE;
            break;
        }
    }
    I2C_Stop();               
}
/******************************************************************************
End of EEPROM_WritePage function
******************************************************************************/

/******************************************************************************
* Function Name:EEPROM_ReadPage
* Description : This function implements EEPROM_ReadPage function.
* Arguments : uint16_t
* Return Value : none
******************************************************************************/
void EEPROM_ReadPage(uint16_t Address)
{
    /* Read 1 page data from eeprom */
    uint8_t i,flag;
    uint8_t Addr_L,Addr_H;
    Addr_L = (uint8_t)Address;
    Addr_H = (uint8_t)(Address>>8);
    g_fEepromError = 0;
    I2C_Start();                            
    flag = I2C_WriteByte(EEPROM_BUS_ADDRESS);     
    flag = I2C_WriteByte(Addr_H);
    flag = I2C_WriteByte(Addr_L);
    if (0 == flag)
    {
        g_fEepromError = TRUE;
        I2C_Stop(); 
        return;
            
    }
    I2C_Start();
    flag = I2C_WriteByte(EEPROM_BUS_ADDRESS|0x01);
    if (0 == flag)
    {
        g_fEepromError = TRUE;
        I2C_Stop(); 
        return;
            
    }
    for(i = 0; i < EEPROM_PAGE_SIZE-1; i++)
    {
        g_aDataBuffer[i] = I2C_ReadByte(ACK); 
    }
    g_aDataBuffer[i]= I2C_ReadByte(NACK);
    I2C_Stop(); 
}
/******************************************************************************
End of EEPROM_ReadPage function
******************************************************************************/
